package PureMessage::Manager::Response::Quarantine::Digest;
use strict;
use PureMessage::Manager::Response::Quarantine;
use PureMessage::Manager::Request qw(%in);
use PureMessage::Manager::Response::Services;
use PureMessage::Manager::Lang;
use PureMessage::Manager::Config qw(%config %web_config);
use PureMessage::Version qw($VERSION $PREFIX);
use ActiveState::Config qw(path_node path_nodes path_children path_value path_values);
use PureMessage::QDigest;
use Exporter;
use HTML::Entities;

our  @ISA = ('PureMessage::Manager::Response::Quarantine','Exporter');
our @EXPORT = qw( );


sub new {
    my $proto = shift;
    my $self = $proto->SUPER::new(@_);

    return $self;
}


our %commands = (
    'pmx-qdigest' => {LABEL=>'Send quarantine digests'},
    'pmx-qdigest-approve' => {LABEL=>'Deliver approved messages'},
    'pmx-qdigest-expire' => {LABEL=>'Expire pending digests'},
);

our %cron_defaults = (
    'pmx-qdigest' => {
        cmd=>$config{prefix}.'/bin/pmx-qdigest --quiet',
        t_time=>15,
        granularity=>'hour',
    },
    'pmx-qdigest-approve' => {
        cmd=>$config{prefix}.'/bin/pmx-qdigest-approve --deliver',
        t_time=>5,
        granularity=>'min',
    },
    'pmx-qdigest-expire' => {
        cmd=>$config{prefix}.'/bin/pmx-qdigest-expire --quiet',
        t_time=>4,
        granularity=>'hour',
    },
);
                    


    my $parser;
    my %parseopts;
    my $root;
    sub load_conf {
        my $self = shift;
        %parseopts = @_;
	require PureMessage::QDigest;
	eval {
            $parser = PureMessage::QDigest::config(%parseopts);
            $root = $parser->root;
	};
	if ($@) {
	    if ($parseopts{writable}) {
                if ($@ =~ /Permission\s*denied/) {
                    $self->raise_error('error_openw_conf_perm');
                }
                else {
                    $self->raise_error('error_openw_conf_unknown',$@);
                }
	    }
	    else {
                $self->raise_error('error_open_conf_unknown',$@);
	    }
        }
    }
    
    sub conf_is_opened {
        my $self = shift;
        return (defined $parser);
    }
    sub get_conf_root {
        my $self = shift;
        return $parser->root unless $self->error;
    }
    sub conf_descend_to {
        my $self = shift;
        $root = $_[0];
    }
    sub get_conf_children {
        my $self = shift;
        my @children = path_children($root,@_);
        if (@children) {
                return @children;
        } else {
                $self->raise_error('error_node_not_found',$_[0]) 
                        unless $self->error;
                return;
        }
        return path_children($root, @_) or $self->raise_error('error_node_not_found',$_[0]) unless $self->error;
    }   
    sub get_conf_hash {
        my $self = shift;
        return $parser->as_hash unless $self->error;
    }
    sub get_conf_node {
        my $self = shift;
        return path_node($root, @_) or $self->raise_error('error_node_not_found',$_[0]) unless $self->error;
    }
    sub conf_value {
        my $self = shift;
        return path_value($root, @_) or $self->raise_error('error_node_not_found',$_[0]) unless $self->error;
    }
    sub conf_values {
        my $self = shift;
        return path_values($root, @_) unless $self->error;
    }
    sub get_conf_values {
        my $self = shift;
        return $self->conf_values(@_);
    }
    sub delete_conf_node {
        my $self = shift;
        my $parent_path = shift;
        my $name = shift;
        my $parent = $self->get_conf_node($parent_path);
        my $iter = scalar $parent->children();
        while (my $node = $iter->next()) {
            if  ($node->groupclass) {
                return $iter->delete  if $node->groupclass eq $name;
            }
            elsif ($node->name) {
                return $iter->delete if $node->name eq $name;
            }
            else {
                # do nothing - placeholder?
            } 
        }
        return undef;
    }  
    sub create_conf_node {
        my $self = shift;
        my $path = shift;
        my $type = shift;
        return path_node($root,$path,create=>$type);
    }
    sub set_conf_groupclass {
        my $self = shift;
        my $node = path_node($root,$_[0]) or $self->raise_error('error_node_not_found',encode_entities($_[0],'&<>"\/\''));
        return $node->groupclass($_[1]) unless $self->error;
        return undef;
    }
    sub set_conf_values {
        my $self = shift;
        eval {
            $self->conf_values(@_);
        };
        if ($@) {
            if ($@ =~ /validation failed: invalid (\w+)/) {
                $self->raise_error('error_'.$1.'_validation',$_[1]);
            }
            else {
                $self->raise_error('error_set_conf_value',$@);
            }
        }   
    }
    
    sub path2code {
        my $self = shift;
        $_[0] =~ tr/#./__/;
        return $_[0];
    }
    
    sub set_conf_value {
        my $self = shift;
        eval {
            $self->conf_value(@_);
        };
        if ($@) {
            if ($@ =~ /validation failed: invalid (\w+)/) {
                $self->raise_error('error_'.$1.'_validation',$_[1]);
            }
            else {
                $self->raise_error('error_set_conf_value',encode_entities($@,'&<>"\/\''));
            }
        }
    }
    sub get_conf_value {
        my $self = shift;
        return $self->conf_value(@_);
    }
    
    sub get_conf_value_formatted {
        my $self = shift;
        return path_node($parser->root, @_)->value_formatted unless $self->error();
    }     
    sub save_conf {
        my $self = shift;
        $parser->save if $parseopts{writable} and !$self->error();
    }

# strip leading and trailing whitespace
sub strip {
        my $self = shift;
    (my $s = shift) =~ s/\s*(.*?)\s*/$1/;
    return $s;
}

# text inputs
sub reload_text_inputs {
        my $self = shift;
    my $params = shift;
    for my $element (@_) {
        $params->{$element} = html_escape($in{$element}) if exists $in{$element};
    }
}

# select inputs
sub reload_select_inputs {
        my $self = shift;
  my $params = shift;  
  for my $element (@_) {
        for my $node (@{$params->{$element}}) {
            $node->{SEL} = '' if exists $in{$element};
            $node->{SEL} = 'selected' if exists $in{$element} and $in{$element} eq  $node->{VALUE};
        }
    }  
}



# try to detect an unfriendly crontab
# basically we only want to parse "friendly" cron entries
# resembling those created in the manager.
# anything else comes up as 'unknown' and is not editable
sub get_crontime {
        my $self = shift;
    my $granularity = shift;
    my %opts = @_;

    $self->raise_error('get_crontime_missing_parameter'), return undef unless $granularity and %opts;
    
    return undef if ($granularity eq 'hour') and (not exists $opts{min} or $opts{min} != 0);
    
    # make sure we only have time field
    for (keys %opts) {
        unless ($granularity eq 'hour' and $_ eq 'min') {
            return undef if $_ ne $granularity and $_ ne 'cmd';
        }
    }
    
    # try to parse the time field
    return parse_crontime($opts{$granularity}, $granularity);
}

sub parse_crontime($$) {
        my $self = shift;
    my $times = shift;
    my $granularity = shift;
    if ($granularity eq 'hour') {
        return undef unless $times and $times =~ /\A\d+(?:,\d+)*\z/;
        return $times;
    }
    elsif ($granularity eq 'min') {
        # $times with be undef if there is a * in the crontab
        # this means run every minute.
        return 1 if !$times;
        return undef unless $times =~ /\A\d+(?:,\d+)*\z/;
        my @times = split ',', $times;
        scalar @times > 2 or return undef;
        $times[0] == 0 or return undef;
        return $times[1]; 
    }
    return undef;
}

# validate user-inputed cron entries
sub validate_crontime($$) {
        my $self = shift;
    my $times = shift;
    my $granularity = shift;
    
    if ($granularity eq 'hour') {
        # clear leading and trailing whitespace
        $times = strip($times);
        # accept spaces instead of commas
        $times =~ s/\s+/,/g unless $times =~ /,/;
        # clear out any whitespace
        $times =~ s/\s+//g;
        # we should now have a comma-delimited list    
        return undef unless $times =~ /\A\d+(?:,\d+)*\z/;
        for (split ',', $times) {
            return undef if $_ > 23;
        }
        return $times;
    }
    elsif ($granularity eq 'min') {
        # clear out any whitespace
        $times =~ s/\s*//g;
        # must be int
        ($times =~ /\A\d+\z/) or return undef;
        # must be 0 < x <= 30
        ($times > 0) or return undef;
        ($times <= 30) or return undef;
        # create a cron string
        return '*' if $times==1;
        my $out = '0';
        for (my $i = $times; $i < 60; $i += $times) {
           $out .= ','.$i;
        }
        return $out;
    }
    return undef;
}

1;

