package PureMessage::Quarantine;

use strict;
use base 'PureMessage::MessageStore';
use PureMessage::Config qw(%config);

our $VERSION = '6.401';

sub new {
    my($class, $spec, %opts) = @_;
    $spec ||= $config{quarantine} || do {
	my $qdir = $config{quarantine_dir};
	my $qtype;
        if ($opts{force_type}) {
            $qtype = delete $opts{force_type};
        } else {
            $qtype = $config{quarantine_type} ||= "dir";
        }
        $qdir ? "$qtype:$qdir" : undef;
    };
    $opts{logger} ||= do {
	require PureMessage::Logger;
        PureMessage::Logger->new(ident => "quarantine");
    };
    return $class->SUPER::new($spec, %opts);
}

1;

__END__

=head1 NAME

PureMessage::Quarantine - The quarantine message store

=head1 SYNPOSIS

  require PureMessage::Quarantine;

  my $q = PureMessage::Quarantine->new;
  print $q->message_count, " messages\n";

=head1 DESCRIPTION

The C<PureMessage::Quarantine> is a subclass of
C<PureMessage::MessageStore>.  It has a simplified constructor in that
it by default will pick up the message store location from the
C<quarantine_dir> setting in the F<pmx.conf> file.

For documentation of the methods implemented see
L<PureMessage::MessageStore>.

=head1 SEE ALSO

L<PureMessage::MessageStore>

