#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use pmx_user;
use PureMessage::MIME::Parser;
use PureMessage::Logger;
use PureMessage::QUser;

require PureMessage::Quarantine;
require PureMessage::QDigest;

my $qtype = $PureMessage::Config::config{quarantine_type};
my $qdir = $PureMessage::Config::config{quarantine_dir};
my %config = PureMessage::QDigest::config()->as_hash;
my $mstore = PureMessage::Quarantine->new("$qtype:$qdir", connect => 1);

my $from = '';
my $digest = '';
my $auto_submitted = '';
my %requested;
my $logger = PureMessage::Logger->new(ident => 'qdigest');

my $mp = PureMessage::MIME::Parser->new;
$mp->decode_te(1);
$mp->handler(part_start => sub {
    my ($p, $hdrs, $ct, $ct_param, $te) = @_;
    if (!$digest and $hdrs =~ /^To:.*\+([0-9a-f]{32})\"?[@%]/msi) {
	$digest = $1;
    }
    if (!$from and $hdrs =~ /^From: (.*)/mi) {
	$from = $1;
    }
    if (!$auto_submitted and $hdrs =~ /^Auto\-Submitted: (.*)/mi) {
	$auto_submitted = $1;
    }
});
my $buf = '';
$mp->handler(data => sub {
    my ($p, $data) = @_;
    $buf .= $data;
    $requested{$_}++ for PureMessage::QDigest::extract_ids($buf);
    $buf =~ s/.*\s+//; # strip any words we know we've seen
});
while (<>) {
    $mp->parse($_);
}
$mp->eof;

$logger->log("INFO", "Not spam request from '$from'");

# Compliant with rfc3834.  
# Do not process digest emails that were auto-submitted.
# ie. out of office replies, change of address, challenges, etc.
if ($auto_submitted && ($auto_submitted !~ /^no\s*$/i)) {
    $logger->log("INFO", "Digest reply auto-submitted, not processing");
    exit 0;
}

# Log an error if the MD5 digest wasn't found.
unless (length $digest) {
    $logger->log("ERR", "not spam $from: could not extract MD5 digest");
    exit;
}

my $info = PureMessage::QDigest::retrieve_pending($digest) or do {
    $logger->log("ERR", "not spam $from: digest id $digest does not exist");
    exit;
};

unless (%requested) {
    $logger->log("ERR", "not spam $from: no digest IDs extracted from message.");
    exit;
}

# Forward those messages which are requested, and which are allowed to be
# released by this digest. Warn on illegal values.
my @forwarded;
for my $id (keys %requested) {
    my $dec;
    # decode_id can die if the ID is invalid.  This can happen 
    # if a non-digest ID is mistakenly extracted from the message.
    eval { $dec = PureMessage::QDigest::decode_id($id, $mstore) };
    if ($@ || !defined $dec) {
        $logger->log("WARNING", "digest ID $id is invalid");
        next;
    }
    next unless $info->{id}{$dec};
    push @forwarded, $dec;
}

# Barf if none of the quarantine ids were valid. This is most likely somebody
# forging an email.
unless (@forwarded) {
    my @req = keys %requested;
    $logger->log("ERR", "not spam $from: no requested IDs (@req) belong to digest '$digest'");
    exit;
}

my $user = PureMessage::QUser->new($from, $logger);
$user->not_spam_msg(@forwarded) if ($user);

__END__

=head1 NAME

pmx-qdigest-not-spam

=head1 SYNOPSIS

   # As invoked by sendmail to parse not spam requests from users. Reads email
   # on standard input.
   | pmx-qdigest-not-spam

=head1 SEE ALSO

L<pmx-qdigest>, L<pmx-qdigest-expire>, L<pmx-queue>

=head1 COPYRIGHT

Copyright (C) 2000-2014 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
